package fr.graphit.bootstrap;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.log4j.Logger;

import com.google.inject.ConfigurationException;
import com.google.inject.Inject;
import com.google.inject.Injector;
import com.google.inject.Key;
import com.google.inject.Provider;
import com.google.inject.name.Names;

import fr.graphit.core.api.ModelService;
import fr.graphit.core.api.exception.database.DatabaseException;
import fr.graphit.core.database.DatabaseHelper;

/** Default implementation of BootstrapService
 * @author Thomas Clarisse
 * @see BootstrapService
 */
public class BootstrapServiceImpl implements BootstrapService {

	private static final Logger logger = Logger.getLogger(BootstrapService.class);
	
	private Provider<Connection> connectionProvider;
	private Injector injector;
	private DatabaseHelper helper;
	
	@Inject
	public BootstrapServiceImpl(Injector injector, Provider<Connection> connectionProvider, DatabaseHelper helper) {
		this.connectionProvider = connectionProvider;
		this.injector = injector;
		this.helper = helper;
	}
	
	@Override
	public void propertiesCheck(List<String> properties) throws RuntimeException {
		
		logger.info("Checking properties : " + properties.toString() + " ...");
		
		try {
			for(String property : properties) {
				injector.getInstance(Key.get(String.class, Names.named(property)));
			}
		}
		catch(ConfigurationException e) {
			if(logger.isDebugEnabled()) {
				logger.debug(ExceptionUtils.getStackTrace(e));
			}
			throw new RuntimeException("Error during property check : " + e.getMessage());
		}
		
		logger.info("Properties checked.");
		
	}
	
	
	
	@Override
	public void schemaConsistencyCheck() throws RuntimeException {
		
		logger.info("Checking schema consistency...");

		Connection connection = connectionProvider.get();
		if(connection!=null) {

			try {

                Statement createStatement = connection.createStatement();
                createStatement.execute("SET FOREIGN_KEY_CHECKS=0");

                logger.debug("Creating if needed tables");
				for(String query : this.queries) {
                    logger.debug(query);
                    createStatement.execute(query);
                    logger.debug("Executed");
                }
                logger.debug("Tables created or already existing.");

                createStatement.execute("SET FOREIGN_KEY_CHECKS=1");

				connection.commit();
			}
			catch (SQLException e) {
				throw new RuntimeException(e.getMessage());
			}
			finally {
				try {
					helper.closeConnection(connection);
				}
				catch (DatabaseException e) {
					if(logger.isDebugEnabled()) {
						logger.debug(ExceptionUtils.getStackTrace(e));
					}
					throw new RuntimeException("Unable to close connection.");
				}
			}
		}
		else {
			throw new RuntimeException("Unable to connect to database.");
		}

		logger.info("Schema consistency checked.");
		
	}

	@Override
	public void modelCheck() throws RuntimeException {

		logger.info("Initiating types and properties...");
		try {
			injector.getInstance(ModelService.class);
			logger.info("Types and properties initiated.");
		}
		catch(Exception e) {
			if(logger.isDebugEnabled()) {
				logger.debug(ExceptionUtils.getStackTrace(e));
			}
			throw new RuntimeException("Unable to load types and properties " + e.getMessage());
		}
		
	}

    private List<String> queries = Arrays.asList("CREATE TABLE IF NOT EXISTS edges (" +
            "  id_parent int(11) NOT NULL," +
            "  id_child int(11) NOT NULL," +
            "  PRIMARY KEY (id_parent,id_child)," +
            "  KEY fk_mother_product (id_parent)," +
            "  KEY fk_daughter_product (id_child)," +
            "  CONSTRAINT fk_daughter_product FOREIGN KEY (id_child) REFERENCES nodes (id) ON DELETE NO ACTION ON UPDATE NO ACTION," +
            "  CONSTRAINT fk_mother_product FOREIGN KEY (id_parent) REFERENCES nodes (id) ON DELETE NO ACTION ON UPDATE NO ACTION" +
            ") ENGINE=InnoDB DEFAULT CHARSET=utf8;",

            "CREATE TABLE IF NOT EXISTS nodes (" +
                    "  id int(11) NOT NULL AUTO_INCREMENT," +
                    "  name varchar(100) NOT NULL," +
                    "  id_type int(11) NOT NULL," +
                    "  PRIMARY KEY (id)," +
                    "  UNIQUE KEY name_UNIQUE (name)," +
                    "  KEY fk_node_type (id_type)," +
                    "  CONSTRAINT fk_node_type FOREIGN KEY (id_type) REFERENCES type_definition (id) ON DELETE NO ACTION ON UPDATE NO ACTION" +
                    ") ENGINE=InnoDB AUTO_INCREMENT=217 DEFAULT CHARSET=utf8;",

            "CREATE TABLE IF NOT EXISTS property_definition (" +
                    "  id int(11) NOT NULL AUTO_INCREMENT," +
                    "  name varchar(45) CHARACTER SET latin1 DEFAULT NULL," +
                    "  component varchar(45) CHARACTER SET latin1 DEFAULT NULL," +
                    "  value_list varchar(500) CHARACTER SET latin1 DEFAULT NULL," +
                    "  PRIMARY KEY (id)" +
                    ") ENGINE=InnoDB AUTO_INCREMENT=9 DEFAULT CHARSET=utf8;",

            "CREATE TABLE IF NOT EXISTS property_values (" +
                    "  id_node int(11) NOT NULL," +
                    "  id_property int(11) NOT NULL," +
                    "  string_value varchar(100) DEFAULT NULL," +
                    "  date_value datetime DEFAULT NULL," +
                    "  text_value text," +
                    "  boolean_value tinyint(1) DEFAULT NULL," +
                    "  integer_value int(11) DEFAULT NULL," +
                    "  PRIMARY KEY (id_node,id_property)," +
                    "  KEY fk_property_node (id_node)," +
                    "  KEY fk_property_property (id_property)," +
                    "  CONSTRAINT fk_property_node FOREIGN KEY (id_node) REFERENCES nodes (id) ON DELETE NO ACTION ON UPDATE NO ACTION," +
                    "  CONSTRAINT fk_property_property FOREIGN KEY (id_property) REFERENCES property_definition (id) ON DELETE NO ACTION ON UPDATE NO ACTION" +
                    ") ENGINE=InnoDB DEFAULT CHARSET=utf8;",

            "CREATE TABLE IF NOT EXISTS settings (" +
                    "  code varchar(45) CHARACTER SET latin1 NOT NULL," +
                    "  value varchar(45) CHARACTER SET latin1 DEFAULT NULL," +
                    "  PRIMARY KEY (code)" +
                    ") ENGINE=InnoDB DEFAULT CHARSET=utf8;",

            "CREATE TABLE IF NOT EXISTS type_composition (" +
                    "  id_type int(11) NOT NULL," +
                    "  id_property int(11) NOT NULL," +
                    "  PRIMARY KEY (id_type,id_property)," +
                    "  KEY fk_type_property_composition (id_property)," +
                    "  KEY fk_property_type_composition (id_type)," +
                    "  CONSTRAINT fk_property_type_composition FOREIGN KEY (id_type) REFERENCES type_definition (id) ON DELETE NO ACTION ON UPDATE NO ACTION," +
                    "  CONSTRAINT fk_type_property_composition FOREIGN KEY (id_property) REFERENCES property_definition (id) ON DELETE NO ACTION ON UPDATE NO ACTION" +
                    ") ENGINE=InnoDB DEFAULT CHARSET=utf8;",

            "CREATE TABLE IF NOT EXISTS type_definition (" +
                    "  id int(11) NOT NULL AUTO_INCREMENT," +
                    "  name varchar(45) CHARACTER SET latin1 DEFAULT NULL," +
                    "  color varchar(45) CHARACTER SET latin1 DEFAULT NULL," +
                    "  shape varchar(45) CHARACTER SET latin1 DEFAULT NULL," +
                    "  PRIMARY KEY (id)," +
                    "  UNIQUE KEY name_UNIQUE (name)" +
                    ") ENGINE=InnoDB AUTO_INCREMENT=6 DEFAULT CHARSET=utf8;",

            "CREATE TABLE IF NOT EXISTS views (" +
                    "  id int(11) NOT NULL AUTO_INCREMENT," +
                    "  name varchar(100) CHARACTER SET latin1 DEFAULT NULL," +
                    "  PRIMARY KEY (id)," +
                    "  UNIQUE KEY name_UNIQUE (name)" +
                    ") ENGINE=InnoDB AUTO_INCREMENT=9 DEFAULT CHARSET=utf8;",

            "CREATE TABLE IF NOT EXISTS views_composition (" +
                    "  id_view int(11) NOT NULL," +
                    "  id_node int(11) NOT NULL," +
                    "  PRIMARY KEY (id_view,id_node)," +
                    "  KEY fk_subgraph_item (id_node)," +
                    "  KEY fk_view_node (id_node)," +
                    "  KEY fk_view_view (id_view)," +
                    "  CONSTRAINT fk_view_node FOREIGN KEY (id_node) REFERENCES nodes (id) ON DELETE NO ACTION ON UPDATE NO ACTION," +
                    "  CONSTRAINT fk_view_view FOREIGN KEY (id_view) REFERENCES views (id) ON DELETE NO ACTION ON UPDATE NO ACTION" +
                    ") ENGINE=InnoDB DEFAULT CHARSET=utf8;",

            "INSERT INTO settings VALUES ('DEFAULT_COLOR','lightskyblue1'),('DEFAULT_SHAPE','oval');",

            "INSERT INTO type_definition VALUES (1,'Default','khaki1','rectangle');"

            );
}
