package fr.graphit.core.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.log4j.Logger;

import com.google.inject.Inject;
import com.google.inject.Injector;

import fr.graphit.core.api.ModelDAO;
import fr.graphit.core.api.exception.database.DatabaseException;
import fr.graphit.core.api.exception.model.TypeNotFoundException;
import fr.graphit.core.api.exception.model.ViewNotFoundException;
import fr.graphit.core.api.exception.node.NameAlreadyInUseException;
import fr.graphit.core.api.exception.node.NodeNotFoundException;
import fr.graphit.core.api.exception.node.UnableToRetrieveNodesException;
import fr.graphit.core.api.model.Node;
import fr.graphit.core.api.model.Property;
import fr.graphit.core.api.model.Type;
import fr.graphit.core.api.model.View;
import fr.graphit.core.database.DatabaseErrorMessages;

public class ModelDAOImpl extends AbstractDAO implements ModelDAO {

	private static final Logger logger = Logger.getLogger(ModelDAO.class);
	
	private List<View> cacheViews;
	private List<Type> cacheTypes;
	private Map<String, String> cacheSettings;
	private Map<Integer, List<Property>> cacheProperties;
	
	@Inject
	public ModelDAOImpl(Injector injector) {
		super(injector);
	}

	@Override
	public List<Type> getTypes() throws DatabaseException {
		
		logger.debug("Starting to list all types...");
		
		cacheTypes = new ArrayList<Type>();
		
		Connection connection = getConnection();
		if(connection!=null) {
			logger.debug("Connection opened");
			try {
				String statement = "select * from " + TYPE_DEFINITION + ";";
				PreparedStatement selectStatement = connection.prepareStatement(statement);
				
				logger.debug("Query to execute : " + selectStatement.toString());
				ResultSet typesResultSet = selectStatement.executeQuery();
				logger.debug("Query executed");
				while(typesResultSet.next()) {
					Type type = injector.getInstance(Type.class);
					type.setColor(typesResultSet.getString("color"));
					type.setId(typesResultSet.getInt("id"));
					type.setName(typesResultSet.getString("name"));
					type.setShape(typesResultSet.getString("shape"));
					cacheTypes.add(type);
				}
				logger.debug("End of execution");
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			finally {
				logger.debug("Closing connection...");
				getDatabaseHelper().closeConnection(connection);
			}
		}
		logger.debug("End of listing");
		logger.debug("Returning " + cacheTypes.size() + " results.");
		return cacheTypes;
	}

	@Override
	public Type getType(Integer id) throws DatabaseException, TypeNotFoundException {
		
		logger.debug("Starting to retrieve type " + id + " ...");
		
		if(cacheTypes != null) {
			for (Type type : cacheTypes) {
				if(type.getId().equals(id)) {
					logger.debug("Found in cache, returning type.");
					return type;
				}
			}
		}
		else {
			cacheTypes = new ArrayList<Type>();
		}
		
		Type type = null;
		
		Connection connection = getConnection();
		if(connection!=null) {
			logger.debug("Connection opened");
			try {
				
				String statement = "select * from " + TYPE_DEFINITION + " where id = ?;";
				PreparedStatement selectStatement = connection.prepareStatement(statement);
				selectStatement.setInt(1, id);
				
				logger.debug("Query to execute : " + selectStatement.toString());
				ResultSet itemsResultSet = selectStatement.executeQuery();
				logger.debug("Query executed");
				if(itemsResultSet.next()) {
					type = injector.getInstance(Type.class);
					type.setColor(itemsResultSet.getString("color"));
					type.setId(itemsResultSet.getInt("id"));
					type.setName(itemsResultSet.getString("name"));
					type.setShape(itemsResultSet.getString("shape"));
				}
				else {
					throw new TypeNotFoundException();
				}
				logger.debug("End of execution");
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			finally {
				logger.debug("Closing connection...");
				getDatabaseHelper().closeConnection(connection);
			}
		}
		logger.debug("Returning " + type);
		cacheTypes.add(type);
		return type;
	}
	
	@Override
	public List<Property> getTypeProperties(Type type) throws DatabaseException, TypeNotFoundException {
		
		logger.debug("Starting to retrieve properties of type " + type.getName() + " ...");
		
		this.getType(type.getId());
		
		if(cacheProperties != null) {
			for (Integer typeId : cacheProperties.keySet()) {
				if(type.getId().equals(typeId)) {
					logger.debug("Found in cache, returning properties.");
					return cacheProperties.get(typeId);
				}
			}
		}
		else {
			cacheProperties = new HashMap<Integer, List<Property>>();
		}
		
		List<Property> properties = new ArrayList<Property>();
				
		Connection connection = getConnection();
		if(connection!=null) {
			logger.debug("Connection opened");
			try {
				
				String statement = "select pd.* from " + TYPE_COMPOSITION + " tc, "+ PROPERTY_DEFINITION +" pd where tc.id_type = ? and pd.id = tc.id_property;";
				PreparedStatement selectStatement = connection.prepareStatement(statement);
				selectStatement.setInt(1, type.getId());
				
				logger.debug("Query to execute : " + selectStatement.toString());
				ResultSet propertiesResultSet = selectStatement.executeQuery();
				logger.debug("Query executed");
				
				while(propertiesResultSet.next()) {
					
					String component = propertiesResultSet.getString("component");
					List<String> listValues = null;
					if(component.equals("LIST")) {
						listValues = Arrays.asList(propertiesResultSet.getString("component").split(","));
					}
					
					String sqlType = "";
					if(component.equals("STRING")) {
						sqlType = "string_value";
					}
					else if(component.equals("TEXT")) {
						sqlType = "text_value";
					}
					else if(component.equals("INTEGER")) {
						sqlType = "integer_value";
					}
					else if(component.equals("BOOLEAN")) {
						sqlType = "boolean_value";
					}
					else if(component.equals("DATE")) {
						sqlType = "date_value";
					}
					
					Property property = new Property(propertiesResultSet.getInt("id"),
							propertiesResultSet.getString("name"), component.equals("LIST"), listValues, sqlType);
					
					properties.add(property);
					
				}
				
				logger.debug("End of execution");
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			finally {
				logger.debug("Closing connection...");
				getDatabaseHelper().closeConnection(connection);
			}
		}
		
		cacheProperties.put(type.getId(),properties);
		
		logger.debug("Returning " + properties);

		return properties;
		
	}

	@Override
	public List<View> getViews() throws DatabaseException {

		logger.debug("Starting to list all views...");
		
		cacheViews = new ArrayList<View>();
		
		Connection connection = getConnection();
		if(connection!=null) {
			logger.debug("Connection opened");
			try {
				String statement = "select * from " + VIEWS + ";";
				PreparedStatement selectStatement = connection.prepareStatement(statement);
				
				logger.debug("Query to execute : " + selectStatement.toString());
				ResultSet itemsResultSet = selectStatement.executeQuery();
				logger.debug("Query executed");
				while(itemsResultSet.next()) {
					View view = injector.getInstance(View.class);
					view.setId(itemsResultSet.getInt("id"));
					view.setName(itemsResultSet.getString("name"));
					cacheViews.add(view);
				}
				logger.debug("End of execution");
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			finally {
				logger.debug("Closing connection...");
				getDatabaseHelper().closeConnection(connection);
			}
		}
		logger.debug("End of listing");
		logger.debug("Returning " + cacheViews.size() + " results.");
		return cacheViews;
		
	}

	@Override
	public View getView(Integer id) throws DatabaseException, ViewNotFoundException {
		
		logger.debug("Starting to retrieve view " + id + " ...");
		
		if(cacheViews != null) {
			for(View view : cacheViews) {
				if(view.getId().equals(id)) {
					logger.debug("Found in cache, returning view.");
					return view;
				}
			}
		}
		else {
			cacheViews = new ArrayList<View>();
		}
		
		View candidate = null;
		
		Connection connection = getConnection();
		if(connection!=null) {
			logger.debug("Connection opened");
			try {
				String statement = "select * from " + VIEWS + " where id = ?;";
				PreparedStatement selectStatement = connection.prepareStatement(statement);
				selectStatement.setInt(1, id);
				
				logger.debug("Query to execute : " + selectStatement.toString());
				ResultSet itemsResultSet = selectStatement.executeQuery();
				logger.debug("Query executed");
				if(itemsResultSet.next()) {
					
					candidate = injector.getInstance(View.class);
					candidate.setId(itemsResultSet.getInt("id"));
					candidate.setName(itemsResultSet.getString("name"));
					
				}
				else {
					throw new ViewNotFoundException();
				}
				logger.debug("End of execution");
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			finally {
				logger.debug("Closing connection...");
				getDatabaseHelper().closeConnection(connection);
			}
		}
		logger.debug("Returning " + candidate);
		cacheViews.add(candidate);
		return candidate;
		
	}

	@Override
	public View addNodeToView(Node node, View target) throws DatabaseException, ViewNotFoundException, NodeNotFoundException {
		
		logger.debug("Adding node to view...");
				
		this.getView(target.getId());
		try {
			this.getNodesService().getNodeById(node.getId());
		}
		catch (UnableToRetrieveNodesException e) {
			throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
		}
		
		Connection connection = getConnection();
		if(connection!=null) {
			logger.debug("Connection opened");
			try {

				String statement = "INSERT INTO " + VIEWS_COMPOSITION + " values(?, ?);";

				PreparedStatement updateStatement = connection.prepareStatement(statement);
				updateStatement.setInt(1, target.getId());
				updateStatement.setInt(2, node.getId());

				logger.debug("Query to execute : " + updateStatement.toString());
				updateStatement.executeUpdate();
				logger.debug("Query executed");

				connection.commit();
				logger.debug("Commit done");
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			
			finally {
				logger.debug("Closing connection...");
				getDatabaseHelper().closeConnection(connection);
			}
		}
		logger.debug("Returning view updated");
		return target;
		
	}

	@Override
	public View deleteNodeFromView(Node node, View target) throws DatabaseException, ViewNotFoundException, NodeNotFoundException {
		
		logger.debug("Deleting node from view...");
		
		this.getView(target.getId());
		try {
			this.getNodesService().getNodeById(node.getId());
		}
		catch (UnableToRetrieveNodesException e) {
			throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
		}
		
		Connection connection = getConnection();
		if(connection!=null) {
			logger.debug("Connection opened");
			try {

				String statement = "DELETE FROM " + VIEWS_COMPOSITION + " WHERE id_view = ? AND id_node = ?;";

				PreparedStatement updateStatement = connection.prepareStatement(statement);
				updateStatement.setInt(1, target.getId());
				updateStatement.setInt(2, node.getId());

				logger.debug("Query to execute : " + updateStatement.toString());
				updateStatement.executeUpdate();
				logger.debug("Query executed");

				connection.commit();
				logger.debug("Commit done");
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			
			finally {
				logger.debug("Closing connection...");
				getDatabaseHelper().closeConnection(connection);
			}
		}
		logger.debug("Returning view updated");
		return target;
		
	}

	@Override
	public Map<String, String> getSettings() throws DatabaseException {
		
		logger.debug("Starting to retrieve settings ...");
		
		if(cacheSettings != null) {
			logger.debug("Found in cache, returning view.");
			return cacheSettings;
		}
		
		cacheSettings = new HashMap<String, String>();
		
		Connection connection = getConnection();
		if(connection!=null) {
			logger.debug("Connection opened");
			try {
				String statement = "select * from " + SETTINGS +";";
				PreparedStatement selectStatement = connection.prepareStatement(statement);
				
				logger.debug("Query to execute : " + selectStatement.toString());
				ResultSet itemsResultSet = selectStatement.executeQuery();
				logger.debug("Query executed");
				while(itemsResultSet.next()) {
					cacheSettings.put(itemsResultSet.getString("code"), itemsResultSet.getString("value"));
				}
				logger.debug("End of execution");
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			finally {
				logger.debug("Closing connection...");
				getDatabaseHelper().closeConnection(connection);
			}
		}
		
		logger.debug("Returning " + cacheSettings);
		return cacheSettings;
	}

	@Override
	public void deleteView(View view) throws DatabaseException, ViewNotFoundException {
		
		logger.debug("Starting deletion " + view.getName() + "...");

		Connection connection = getConnection();

		if(connection!=null) {
			logger.debug("Connection opened");
			try {

				// deleting tags

				String statement = "delete from " + VIEWS_COMPOSITION + " where id_view = ?;";

				PreparedStatement deleteStatement = connection.prepareStatement(statement);
				deleteStatement.setInt(1, view.getId());

				logger.debug("Query to execute : " + deleteStatement.toString());
				deleteStatement.executeUpdate();
				logger.debug("Query executed");

				// deleting properties

				statement = "delete from " + VIEWS + " where id = ?;";

				deleteStatement = connection.prepareStatement(statement);
				deleteStatement.setInt(1, view.getId());

				logger.debug("Query to execute : " + deleteStatement.toString());
				deleteStatement.executeUpdate();
				logger.debug("Query executed");

				connection.commit();
				logger.debug("Commit done");
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			finally {
				logger.debug("Closing connection");
				getDatabaseHelper().closeConnection(connection);
			}
		}
	}

	@Override
	public View createView(String name) throws DatabaseException, NameAlreadyInUseException {
		
		logger.debug("Starting creation " + name + "...");

		// TODO verify that name is not taken

		Connection connection = getConnection();
		Integer generatedId = -1;
		if(connection!=null) {
			logger.debug("Connection opened");
			try {

				String statement = "insert into " + VIEWS + " values(0, ?);";

				PreparedStatement createStatement = connection.prepareStatement(statement, Statement.RETURN_GENERATED_KEYS);
				createStatement.setString(1, name);

				logger.debug("Query to execute : " + createStatement.toString());
				createStatement.executeUpdate();
				logger.debug("Query executed");
				ResultSet keys = createStatement.getGeneratedKeys();
				if(keys.next()) {
					generatedId = keys.getInt(1);
				}
				logger.debug("Created key is " + generatedId);

				connection.commit();
				logger.debug("Commit done");
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			
			finally {
				logger.debug("Closing connection");
				getDatabaseHelper().closeConnection(connection);
			}
		}
		logger.debug("Returning view created");

		View newView = null;
		try {
			newView = this.getView(generatedId);
		}
		catch (ViewNotFoundException e) {
			// TODO should never happen but do something
		}

		return newView;
		
	}

	@Override
	public View updateView(View view) throws DatabaseException, ViewNotFoundException {
		
		logger.debug("Starting to update " + view.getName() + "...");
		
		this.getView(view.getId());
		
		this.cacheViews = null;
		
		Connection connection = getConnection();
		if(connection!=null) {
			logger.debug("Connection opened");
			try {

				String statement = "update " + VIEWS + " set name = ? where id = ?;";

				PreparedStatement updateStatement = connection.prepareStatement(statement);
				updateStatement.setString(1, view.getName());
				updateStatement.setInt(2, view.getId());

				logger.debug("Query to execute : " + updateStatement.toString());
				updateStatement.executeUpdate();
				logger.debug("Query executed");

				connection.commit();
				logger.debug("Commit done");
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			
			finally {
				logger.debug("Closing connection...");
				getDatabaseHelper().closeConnection(connection);
			}
		}
		logger.debug("Returning item updated");
		return this.getView(view.getId());
	}

}
