package fr.graphit.core.dao;

import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.log4j.Logger;

import com.google.inject.Inject;
import com.google.inject.Injector;

import fr.graphit.core.api.ModelService;
import fr.graphit.core.api.NodesDAO;
import fr.graphit.core.api.exception.database.DatabaseException;
import fr.graphit.core.api.exception.model.TypeNotFoundException;
import fr.graphit.core.api.exception.model.UnableToRetrieveTypesException;
import fr.graphit.core.api.exception.model.UnableToRetrieveViewsException;
import fr.graphit.core.api.exception.model.ViewNotFoundException;
import fr.graphit.core.api.exception.node.EdgeAlreadyExistsException;
import fr.graphit.core.api.exception.node.EdgeNotFoundException;
import fr.graphit.core.api.exception.node.NameAlreadyInUseException;
import fr.graphit.core.api.exception.node.NodeNotFoundException;
import fr.graphit.core.api.model.Node;
import fr.graphit.core.api.model.Property;
import fr.graphit.core.api.model.Type;
import fr.graphit.core.api.model.View;
import fr.graphit.core.database.DatabaseErrorMessages;

/**
 * 
 * Default implementation of NodesDAO
 * @author Thomas Clarisse
 *
 */
public class NodesDAOImpl extends AbstractDAO implements NodesDAO {

	private static final Logger logger = Logger.getLogger(NodesDAO.class);

	@Inject
	public NodesDAOImpl(Injector injector) {
		super(injector);
	}

	@Override
	public List<Node> getAllNodes(Boolean order) throws DatabaseException {
		
		logger.debug("Starting to list all nodes...");
		
		List<Node> candidates = new ArrayList<Node>();
		
		Connection connection = getConnection();
		if(connection!=null) {
			logger.debug("Connection opened");
			try {
				String statement = "select * from " + NODES + (order?" ORDER BY name ":"") + ";";
				PreparedStatement selectStatement = connection.prepareStatement(statement);

				logger.debug("Query to execute : " + selectStatement.toString());
				ResultSet itemsResultSet = selectStatement.executeQuery();
				logger.debug("Query executed");
				while(itemsResultSet.next()) {
					Node node = injector.getInstance(Node.class);
					node.setId(itemsResultSet.getInt("id"));
					node.setName(itemsResultSet.getString("name"));
					node.setType(getModelService().getType(itemsResultSet.getInt("id_type")));
					candidates.add(node);
				}
				logger.debug("End of execution");
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			catch (UnableToRetrieveTypesException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			catch (TypeNotFoundException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			finally {
				logger.debug("Closing connection...");
				getDatabaseHelper().closeConnection(connection);
			}
		}
		logger.debug("End of listing");
		logger.debug("Returning " + candidates.size() + " results.");
		return candidates;
	}

	@Override
	public Node getNodeById(Integer id) throws NodeNotFoundException, DatabaseException {
		
		logger.debug("Starting to get " + id + "...");
		
		Node candidate = null;
		
		Connection connection = getConnection();
		if(connection!=null) {
			logger.debug("Connection opened");
			try {
				String statement = "select * from " + NODES + " where id = ?;";
				PreparedStatement selectStatement = connection.prepareStatement(statement);
				selectStatement.setInt(1, id);

				logger.debug("Query to execute : " + selectStatement.toString());
				ResultSet nodesResultSet = selectStatement.executeQuery();
				logger.debug("Query executed");
				if(nodesResultSet.next()) {
					candidate = injector.getInstance(Node.class);
					candidate.setId(nodesResultSet.getInt("id"));
					candidate.setName(nodesResultSet.getString("name"));
					candidate.setType(getModelService().getType(nodesResultSet.getInt("id_type")));
				}
				else {
					throw new NodeNotFoundException();
				}
				logger.debug("End of execution");
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			catch (UnableToRetrieveTypesException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			catch (TypeNotFoundException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			finally {
				logger.debug("Closing connection...");
				getDatabaseHelper().closeConnection(connection);
			}
		}
		logger.debug("End of getting " + id);
		logger.debug("Returning " + (candidate==null?"an node":"nothing"));
		return candidate;
	}

	@Override
	public List<Node> searchNodesByName(String name, Boolean strict) throws DatabaseException {
		
		logger.debug("Starting to search " + name + "...");
		
		List<Node> candidates = new ArrayList<Node>();
		
		Connection connection = getConnection();
		if(connection!=null) {
			logger.debug("Connection opened");
			try {
				String statement = "select * from " + NODES + " where name like ? order by name;";

				PreparedStatement selectStatement = connection.prepareStatement(statement);
				if(strict) {
					selectStatement.setString(1, name);
				}
				else {
					selectStatement.setString(1, "%" + name + "%");
				}

				logger.debug("Query to execute : " + selectStatement.toString());
				ResultSet nodesResultSet = selectStatement.executeQuery();
				logger.debug("Query executed");
				while(nodesResultSet.next()) {
					Node node = injector.getInstance(Node.class);
					node.setId(nodesResultSet.getInt("id"));
					node.setName(nodesResultSet.getString("name"));
					node.setType(getModelService().getType(nodesResultSet.getInt("id_type")));
					candidates.add(node);
				}
				logger.debug("End of execution");
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			catch (UnableToRetrieveTypesException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			catch (TypeNotFoundException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			finally {
				logger.debug("Closing connection...");
				getDatabaseHelper().closeConnection(connection);
			}
		}
		logger.debug("End of search " + name);
		logger.debug("Returning " + candidates.size() + " results.");
		return candidates;
	}

	@Override
	public List<Node> searchNodesByView(View view) throws DatabaseException, ViewNotFoundException {
		logger.debug("Starting to search nodes from view " + view.getName() + "...");
		
		ModelService model = injector.getInstance(ModelService.class);
		try {
			model.getView(view.getId());
		}
		catch (UnableToRetrieveViewsException e) {
			if(logger.isDebugEnabled()) {
				logger.debug(ExceptionUtils.getStackTrace(e));
			}
			logger.error("SQL Exception thrown " + e.getMessage());
			throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
		}
		
		List<Node> candidates = new ArrayList<Node>();
		
		Connection connection = getConnection();
		if(connection!=null) {
			logger.debug("Connection opened");
			try {
				String statement = "select n.* from " + NODES + " n, " + VIEWS_COMPOSITION + " v where v.id_node = n.id and v.id_view = ?;";

				PreparedStatement selectStatement = connection.prepareStatement(statement);
				selectStatement.setInt(1, view.getId());

				logger.debug("Query to execute : " + selectStatement.toString());
				ResultSet nodesResultSet = selectStatement.executeQuery();
				logger.debug("Query executed");
				while(nodesResultSet.next()) {
					Node node = injector.getInstance(Node.class);
					node.setId(nodesResultSet.getInt("id"));
					node.setName(nodesResultSet.getString("name"));
					node.setType(getModelService().getType(nodesResultSet.getInt("id_type")));
					candidates.add(node);
				}
				logger.debug("End of execution");
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			catch (UnableToRetrieveTypesException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			catch (TypeNotFoundException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			finally {
				logger.debug("Closing connection...");
				getDatabaseHelper().closeConnection(connection);
			}
		}
		logger.debug("Returning " + candidates.size() + " results.");
		return candidates;
	}

	@Override
	public List<Node> getNodeParents(Node target, Integer level) throws NodeNotFoundException, DatabaseException {
		logger.debug("Starting to list all parents (level " + level + ") of " + target.getName() + "...");
		
		this.getNodeById(target.getId());

		List<Node> candidates = new ArrayList<Node>();

		List<StackElement> childrenStack = new ArrayList<StackElement>();
		childrenStack.add(new StackElement(1, target));

		Connection connection = getConnection();
		if(connection!=null) {
			logger.debug("Connection opened");
			try {
				while(childrenStack.size() > 0) {

					String statement = "select i.* from " + EDGES + " l, " + NODES + " i" +
							" where l.id_parent = i.id" +
							" and l.id_child = ?;";

					PreparedStatement selectStatement = connection.prepareStatement(statement);
					selectStatement.setInt(1, childrenStack.get(0).getNode().getId());

					logger.debug("Query to execute : " + selectStatement.toString());
					ResultSet nodesResultSet = selectStatement.executeQuery();
					logger.debug("Query executed");
					while(nodesResultSet.next()) {
						Node candidate = injector.getInstance(Node.class);
						candidate.setId(nodesResultSet.getInt("id"));
						candidate.setName(nodesResultSet.getString("name"));
						candidate.setType(getModelService().getType(nodesResultSet.getInt("id_type")));
						candidates.add(candidate);
						if(childrenStack.get(0).getLevel() < level) {
							childrenStack.add(new StackElement(childrenStack.get(0).getLevel()+1, candidate));
						}
					}
					logger.debug("End of execution");
					childrenStack.remove(0);
				}
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			catch (UnableToRetrieveTypesException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			catch (TypeNotFoundException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			finally {
				logger.debug("Closing connection...");
				getDatabaseHelper().closeConnection(connection);
			}
		}
		logger.debug("End of listing");
		logger.debug("Returning " + candidates.size() + " results.");
		return candidates;
	}

	@Override
	public List<Node> getNodeChildren(Node target, Integer level) throws NodeNotFoundException, DatabaseException {
		logger.debug("Starting to list all children (level " + level + ") of " + target.getName() + "...");
		
		this.getNodeById(target.getId());
		
		List<Node> candidates = new ArrayList<Node>();

		List<StackElement> parentStack = new ArrayList<StackElement>();
		parentStack.add(new StackElement(1, target));

		Connection connection = getConnection();
		if(connection!=null) {
			logger.debug("Connection opened");
			try {
				while(parentStack.size() > 0) {

					String statement = "select i.* from " + EDGES + " l, " + NODES + " i" +
							" where l.id_child = i.id" +
							" and l.id_parent = ?;";

					PreparedStatement selectStatement = connection.prepareStatement(statement);
					selectStatement.setInt(1, parentStack.get(0).getNode().getId());

					logger.debug("Query to execute : " + selectStatement.toString());
					ResultSet nodesResultSet = selectStatement.executeQuery();
					logger.debug("Query executed");
					while(nodesResultSet.next()) {
						Node candidate = injector.getInstance(Node.class);
						candidate.setId(nodesResultSet.getInt("id"));
						candidate.setName(nodesResultSet.getString("name"));
						candidate.setType(getModelService().getType(nodesResultSet.getInt("id_type")));
						candidates.add(candidate);
						if(parentStack.get(0).getLevel() < level) {
							parentStack.add(new StackElement(parentStack.get(0).getLevel()+1, candidate));
						}
					}
					logger.debug("End of execution");
					parentStack.remove(0);
				}
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			catch (UnableToRetrieveTypesException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			catch (TypeNotFoundException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			finally {
				logger.debug("Closing connection...");
				getDatabaseHelper().closeConnection(connection);
			}
		}
		logger.debug("End of listing");
		logger.debug("Returning " + candidates.size() + " results.");
		return candidates;
	}

	@Override
	public Map<Property, Object> getNodeProperties(Node target) throws NodeNotFoundException, DatabaseException {
		logger.debug("Gathering properties...");

		this.getNodeById(target.getId());
		
		Map<Property, Object> candidates = null;
		Connection connection = getConnection();
		try {

			if(connection!=null) {
				logger.debug("Connection opened");
				candidates = new HashMap<Property, Object>();
				String statement = "select * from " + PROPERTY_VALUES + " where id_node = ?;";
				PreparedStatement selectStatement = connection.prepareStatement(statement);
				selectStatement.setInt(1, target.getId());
				logger.debug("Query to execute : " + selectStatement.toString());
				ResultSet propertiesResultSet = selectStatement.executeQuery();
				logger.debug("Query executed");
				while(propertiesResultSet.next()) {
					Integer propertyId = propertiesResultSet.getInt("id_property");
					Property propertyDefinition = target.getType().getProperty(propertyId);
					if(propertyDefinition != null) {
						candidates.put(propertyDefinition, propertiesResultSet.getString(propertyDefinition.getSqlType()));
					}
				}
				logger.debug("End of execution");
			}

		}
		catch (SQLException e) {
			logger.error("SQL Exception thrown " + e.getMessage());
			throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
		}
		catch (UnableToRetrieveTypesException e) {
			if(logger.isDebugEnabled()) {
				logger.debug(ExceptionUtils.getStackTrace(e));
			}
			logger.error("SQL Exception thrown " + e.getMessage());
			throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
		}
		catch (TypeNotFoundException e) {
			if(logger.isDebugEnabled()) {
				logger.debug(ExceptionUtils.getStackTrace(e));
			}
			logger.error("SQL Exception thrown " + e.getMessage());
			throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
		}
		finally {
			logger.debug("Closing connection...");
			getDatabaseHelper().closeConnection(connection);
		}

		return candidates;
	}

	@Override
	public Node updateNode(Node updatedNode) throws NodeNotFoundException, DatabaseException {
		
		logger.debug("Starting to update " + updatedNode.getName() + "...");
		
		this.getNodeById(updatedNode.getId());
		
		Connection connection = getConnection();
		if(connection!=null) {
			logger.debug("Connection opened");
			try {

				Node oldNode = this.getNodeById(updatedNode.getId());

				String statement = "update " + NODES + " set name = ?, id_type = ? where id = ?;";

				PreparedStatement updateStatement = connection.prepareStatement(statement);
				updateStatement.setString(1, updatedNode.getName());
				updateStatement.setInt(2, updatedNode.getType().getId());
				updateStatement.setInt(3, updatedNode.getId());

				logger.debug("Query to execute : " + updateStatement.toString());
				updateStatement.executeUpdate();
				logger.debug("Query executed");

				if(!oldNode.getType().equals(updatedNode.getType())) {
					logger.debug("Type has changed, deleting properties and recreating...");

					statement = "delete from " + PROPERTY_VALUES + " where id_node = ?;";
					PreparedStatement deleteStatement = connection.prepareStatement(statement);
					deleteStatement.setInt(1, updatedNode.getId());

					logger.debug("Query to execute : " + deleteStatement.toString());
					deleteStatement.executeUpdate();
					logger.debug("Query executed");

					for(Property property : updatedNode.getType().properties()) {

						statement = "insert into " + PROPERTY_VALUES + "(id_node, id_property, " + property.getSqlType() + ") values(?,?,?);";
						updateStatement = connection.prepareStatement(statement);
						updateStatement.setInt(1, updatedNode.getId());
						updateStatement.setInt(2, property.getId());

						if(updatedNode.properties().get(property.getId()) != null) {
							Object newValue = updatedNode.properties().get(property.getId());
							if((property.getList() && property.getListValues().contains(newValue)) || !property.getList()) {
								setVariable(3, property.getSqlType(), newValue, updateStatement);
							}
							else {
								setVariable(3, property.getSqlType(), null, updateStatement);
							}
						}
						else {
							setVariable(3, property.getSqlType(), null, updateStatement);
						}

						logger.debug("Query to execute : " + updateStatement.toString());
						updateStatement.executeUpdate();
						logger.debug("Query executed");

					}


				}
				else {
					logger.debug("Updating properties...");
					for(Property property : updatedNode.getType().properties()) {
						if(updatedNode.properties().get(property.getId()) != null) {
							Object newValue = updatedNode.properties().get(property.getId());
							if((property.getList() && property.getListValues().contains(newValue)) || !property.getList()) {

								statement = "update " + PROPERTY_VALUES + " set " + property.getSqlType() + " = ? where id_node = ? and id_property = ?;";
								updateStatement = connection.prepareStatement(statement);
								setVariable(1, property.getSqlType(), newValue, updateStatement);
								updateStatement.setInt(2, updatedNode.getId());
								updateStatement.setInt(3, property.getId());

								logger.debug("Query to execute : " + updateStatement.toString());
								updateStatement.executeUpdate();
								logger.debug("Query executed");
							}
						}
					}
				}

				connection.commit();
				logger.debug("Commit done");
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			
			finally {
				logger.debug("Closing connection...");
				getDatabaseHelper().closeConnection(connection);
			}
		}
		logger.debug("Returning item updated");
		return this.getNodeById(updatedNode.getId());
	}

	@Override
	public Node createNode(String name, Type type) throws NameAlreadyInUseException, TypeNotFoundException, DatabaseException {
		
		logger.debug("Starting creation " + name + "...");

		// verify that name is not taken
		List<Node> candidates = this.searchNodesByName(name, true);
		if(candidates.size() > 0) {
			throw new NameAlreadyInUseException();
		}

		Connection connection = getConnection();
		Integer generatedId = -1;
		if(connection!=null) {
			logger.debug("Connection opened");
			try {

				String statement = "insert into " + NODES + " values(0, ?, ?);";

				PreparedStatement createStatement = connection.prepareStatement(statement, Statement.RETURN_GENERATED_KEYS);
				createStatement.setString(1, name);
				createStatement.setInt(2, type.getId());

				logger.debug("Query to execute : " + createStatement.toString());
				createStatement.executeUpdate();
				logger.debug("Query executed");
				ResultSet keys = createStatement.getGeneratedKeys();
				if(keys.next()) {
					generatedId = keys.getInt(1);
				}
				logger.debug("Created key is " + generatedId);

				for(Property property : type.properties()) {

					statement = "insert into " + PROPERTY_VALUES + "(id_node, id_property, " + property.getSqlType() + ") values(?,?,?);";
					createStatement = connection.prepareStatement(statement);
					createStatement.setInt(1, generatedId);
					createStatement.setInt(2, property.getId());
					setVariable(3, property.getSqlType(), null, createStatement);

					logger.debug("Query to execute : " + createStatement.toString());
					createStatement.executeUpdate();
					logger.debug("Query executed");

				}

				connection.commit();
				logger.debug("Commit done");
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			
			finally {
				logger.debug("Closing connection");
				getDatabaseHelper().closeConnection(connection);
			}
		}
		logger.debug("Returning item created");

		Node newNode = null;
		try {
			newNode = this.getNodeById(generatedId);
		}
		catch (NodeNotFoundException e) {
			// TODO should never happen but do something
		}

		return newNode;

	}

	@Override
	public void deleteNode(Node deletedNode) throws NodeNotFoundException, DatabaseException {
		logger.debug("Starting deletion " + deletedNode.getName() + "...");

		Connection connection = getConnection();

		if(connection!=null) {
			logger.debug("Connection opened");
			try {

				// deleting tags

				String statement = "delete from " + VIEWS_COMPOSITION + " where id_node = ?;";

				PreparedStatement deleteStatement = connection.prepareStatement(statement);
				deleteStatement.setInt(1, deletedNode.getId());

				logger.debug("Query to execute : " + deleteStatement.toString());
				deleteStatement.executeUpdate();
				logger.debug("Query executed");

				// deleting properties

				statement = "delete from " + PROPERTY_VALUES + " where id_node = ?;";

				deleteStatement = connection.prepareStatement(statement);
				deleteStatement.setInt(1, deletedNode.getId());

				logger.debug("Query to execute : " + deleteStatement.toString());
				deleteStatement.executeUpdate();
				logger.debug("Query executed");

				// deleting links

				statement = "delete from " + EDGES + " where id_parent = ? or id_child = ?;";

				deleteStatement = connection.prepareStatement(statement);
				deleteStatement.setInt(1, deletedNode.getId());
				deleteStatement.setInt(2, deletedNode.getId());

				logger.debug("Query to execute : " + deleteStatement.toString());
				deleteStatement.executeUpdate();
				logger.debug("Query executed");

				// finaly deleting node

				statement = "delete from " + NODES + " where id = ?;";

				deleteStatement = connection.prepareStatement(statement);
				deleteStatement.setInt(1, deletedNode.getId());

				logger.debug("Query to execute : " + deleteStatement.toString());
				deleteStatement.executeUpdate();
				logger.debug("Query executed");

				connection.commit();
				logger.debug("Commit done");
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			finally {
				logger.debug("Closing connection");
				getDatabaseHelper().closeConnection(connection);
			}
		}

	}

	@Override
	public void createEdge(Node parent, Node child) throws NodeNotFoundException, EdgeAlreadyExistsException, DatabaseException {
		logger.debug("Creating link between " + parent.getName() + " and " + child.getName() + "...");

		Connection connection = getConnection();
		if(connection!=null) {
			logger.debug("Connection opened");
			try {
				
				List<Node> checkChildren = this.getNodeChildren(parent, 1);
				for(Node checkChild : checkChildren) {
					if(checkChild.equals(child)) {
						throw new EdgeAlreadyExistsException();
					}
				}
				
				String statement = "insert into " + EDGES + " values(?, ?);";
				PreparedStatement insertStatement = connection.prepareStatement(statement);
				insertStatement.setInt(1, parent.getId());
				insertStatement.setInt(2, child.getId());

				logger.debug("Query to execute : " + insertStatement.toString());
				insertStatement.executeUpdate();
				logger.debug("Query executed");
				connection.commit();
				logger.debug("Commit done");
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			catch (NodeNotFoundException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			finally {
				logger.debug("Closing connection");
				getDatabaseHelper().closeConnection(connection);
			}
		}
		logger.debug("Ending link creation");

	}

	@Override
	public void breakEdge(Node parent, Node child) throws NodeNotFoundException, EdgeNotFoundException, DatabaseException {
		logger.debug("Starting to delete link between " + parent.getName() + " and " + child.getName() + "...");
		Connection connection = getConnection();
		if(connection!=null) {
			logger.debug("Connection opened");
			try {
				String statement = "delete from " + EDGES + " where id_parent = ? and id_child = ?;";
				PreparedStatement deleteStatement = connection.prepareStatement(statement);
				deleteStatement.setInt(1, parent.getId());
				deleteStatement.setInt(2, child.getId());
				logger.debug("Query to execute : " + deleteStatement.toString());
				deleteStatement.executeUpdate();
				logger.debug("Query executed");
				connection.commit();
				logger.debug("Commit done");
			}
			catch (SQLException e) {
				if(logger.isDebugEnabled()) {
					logger.debug(ExceptionUtils.getStackTrace(e));
				}
				logger.error("SQL Exception thrown " + e.getMessage());
				throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
			}
			finally {
				logger.debug("Closing connection");
				getDatabaseHelper().closeConnection(connection);
			}
		}
		logger.debug("End of deletion");

	}
	
	private void setVariable(Integer index, String sqlType, Object value, PreparedStatement preparedStatement) throws DatabaseException {
		try {
			if(sqlType.equals("string_value") || sqlType.equals("text_value")) {
				if(value == null) {
					preparedStatement.setNull(index, Types.VARCHAR);
					return;
				}
				preparedStatement.setString(index, (String)value);
				return;
			}
			if(sqlType.equals("integer_value")) {
				if(value == null) {
					preparedStatement.setNull(index, Types.INTEGER);
					return;
				}
				preparedStatement.setInt(index, (Integer)value);
				return;
			}
			if(sqlType.equals("boolean_value")) {
				if(value == null) {
					preparedStatement.setNull(index, Types.BOOLEAN);
					return;
				}
				preparedStatement.setBoolean(index, (Boolean)value);
				return;
			}
			if(sqlType.equals("date_value")) {
				if(value == null) {
					preparedStatement.setNull(index, Types.DATE);
					return;
				}
				preparedStatement.setDate(index, (Date)value);
				return;
			}
		}
		catch(SQLException e) {
			if(logger.isDebugEnabled()) {
				logger.debug(ExceptionUtils.getStackTrace(e));
			}
			logger.error("Error while preparing statement");
			throw new DatabaseException(DatabaseErrorMessages.UNKNOWN, e);
		}
	}
	
	protected class StackElement {

		private Integer level;
		private Node node;

		public StackElement(Integer level, Node node) {
			this.level = level;
			this.node = node;
		}

		public Integer getLevel() {
			return level;
		}

		public void setLevel(Integer level) {
			this.level = level;
		}

		public Node getNode() {
			return node;
		}

		public void setNode(Node node) {
			this.node = node;
		}

	}


}
