package fr.graphit.web;

import java.io.IOException;
import java.util.Properties;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.log4j.Logger;

import com.google.inject.Singleton;
import com.google.inject.name.Names;
import com.google.inject.servlet.SessionScoped;
import com.google.sitebricks.SitebricksModule;

import fr.graphit.web.api.AddLinkAPIService;
import fr.graphit.web.api.AddNodeToViewAPIService;
import fr.graphit.web.api.DeleteLinkAPIService;
import fr.graphit.web.api.DeleteNodeAPIService;
import fr.graphit.web.api.DeleteNodeFromViewAPIService;
import fr.graphit.web.api.DeleteViewAPIService;
import fr.graphit.web.api.GetLinksAPIService;
import fr.graphit.web.api.GraphAPIService;
import fr.graphit.web.api.NodeAPIService;
import fr.graphit.web.api.SearchAPIService;
import fr.graphit.web.api.graph.provider.GoogleGraphProvider;
import fr.graphit.web.api.graph.provider.GraphProvider;
import fr.graphit.web.api.graph.provider.NativeGraphProvider;
import fr.graphit.web.client.BadBrowserService;
import fr.graphit.web.client.ConnectAndDisconnectService;
import fr.graphit.web.client.IndexService;
import fr.graphit.web.client.ViewListService;
import fr.graphit.web.client.ViewService;
import fr.graphit.web.client.admin.AddService;
import fr.graphit.web.client.admin.AddViewService;
import fr.graphit.web.client.admin.AdminNodesService;
import fr.graphit.web.client.admin.AdminViewsService;
import fr.graphit.web.client.admin.DisplayService;
import fr.graphit.web.client.admin.EditService;
import fr.graphit.web.client.admin.EditViewService;
import fr.graphit.web.client.admin.SearchService;
import fr.graphit.web.client.template.TemplateEngine;
import fr.graphit.web.messages.UIMessages;
import fr.graphit.web.session.SessionHandler;
import fr.graphit.web.uri.URIContext;

/**
 * WebModule is initiating services used in Web client and API
 * @author Thomas Clarisse
 *
 */
public class WebModule extends SitebricksModule {

	private static final Logger logger = Logger.getLogger(WebModule.class);
	
	@Override
	protected void configureSitebricks() {
		
		logger.info("Starting Graphit Web Module...");
		
		at(URIContext.ROOT).serve(IndexService.class);
		at(URIContext.DISPLAY).serve(DisplayService.class);
		at(URIContext.VIEW).serve(ViewService.class);
		at(URIContext.VIEWS).serve(ViewListService.class);
		at(URIContext.SEARCH).serve(SearchService.class);
		at(URIContext.EDIT).serve(EditService.class);
		at(URIContext.ADD).serve(AddService.class);
		at(URIContext.ADMIN_NODES).serve(AdminNodesService.class);
		at(URIContext.ADMIN_VIEWS).serve(AdminViewsService.class);
		at(URIContext.CONNECT).serve(ConnectAndDisconnectService.class);
		at(URIContext.DISCONNECT).serve(ConnectAndDisconnectService.class);
		at(URIContext.ADD_VIEW).serve(AddViewService.class);
		at(URIContext.EDIT_VIEW).serve(EditViewService.class);
		
		// this is use to display configuration if needed
		// must not be activated in production
		//at(URIContext.CONF).serve(DisplayConfigService.class);
		
		at(URIContext.BAD_BROWSER).serve(BadBrowserService.class);
		
		at(URIContext.API_DELETE_LINK).serve(DeleteLinkAPIService.class);
		at(URIContext.API_ADD_LINK).serve(AddLinkAPIService.class);
		at(URIContext.API_GET_LINKS).serve(GetLinksAPIService.class);
		at(URIContext.API_SEARCH).serve(SearchAPIService.class);
		at(URIContext.API_GRAPH).serve(GraphAPIService.class);
		at(URIContext.API_NODE).serve(NodeAPIService.class);
		at(URIContext.API_DELETE).serve(DeleteNodeAPIService.class);
		at(URIContext.API_DELETE_VIEW).serve(DeleteViewAPIService.class);
		at(URIContext.API_ADD_NODE_VIEW).serve(AddNodeToViewAPIService.class);
		at(URIContext.API_DELETE_NODE_VIEW).serve(DeleteNodeFromViewAPIService.class);
		
		
		bind(SessionHandler.class).in(SessionScoped.class);
		bind(TemplateEngine.class).in(Singleton.class);
		
		bind(GraphProvider.class).annotatedWith(Names.named("google")).to(GoogleGraphProvider.class);
		bind(GraphProvider.class).annotatedWith(Names.named("native")).to(NativeGraphProvider.class);
		
		localize(UIMessages.class).usingDefault();
		
		Properties properties = new Properties();
		String currentPath = null;
		try {
			currentPath = "/fr/graphit/conf/app.properties";
			properties.load(this.getClass().getClassLoader().getResourceAsStream(currentPath));
			currentPath = "fr/graphit/conf/custom-app.properties";
			properties.load(this.getClass().getClassLoader().getResourceAsStream(currentPath));
		}
		catch (IOException e) {
			if(logger.isDebugEnabled()) {
				logger.debug(ExceptionUtils.getStackTrace(e));
			}
			logger.error("Unable to load configuration file " + currentPath + " : " + e.getMessage());
		}
		catch (NullPointerException e) {
			if(logger.isDebugEnabled()) {
				logger.debug(ExceptionUtils.getStackTrace(e));
			}
			logger.error("Unable to load configuration file " + currentPath + " : " + e.getMessage());
		}
		finally {
			Names.bindProperties(binder(), properties);
		}
		
		logger.info("Graphit Web Module is started.");
		
	}
	
}
