package fr.graphit.web.api;

import java.util.ArrayList;
import java.util.List;

import com.google.inject.Inject;
import com.google.inject.Injector;
import com.google.inject.Key;
import com.google.inject.name.Named;
import com.google.inject.name.Names;
import com.google.sitebricks.client.transport.Json;
import com.google.sitebricks.headless.Reply;
import com.google.sitebricks.http.Get;
import com.google.sitebricks.http.Post;

import fr.graphit.core.api.ModelService;
import fr.graphit.core.api.NodesService;
import fr.graphit.core.api.exception.model.UnableToRetrieveViewsException;
import fr.graphit.core.api.exception.model.ViewNotFoundException;
import fr.graphit.core.api.exception.node.NodeNotFoundException;
import fr.graphit.core.api.exception.node.UnableToRetrieveNodesException;
import fr.graphit.core.api.model.Node;
import fr.graphit.web.api.graph.GraphBuilder;
import fr.graphit.web.api.reply.ErrorReply;
import fr.graphit.web.api.reply.transport.PNG;
import fr.graphit.web.api.reply.transport.SVG;
import fr.graphit.web.exception.GraphException;
import fr.graphit.web.uri.URIContext;

public class GraphAPIService implements APIService {

	private Integer id_center;
	private Integer id_subgraph;
	private Integer parents_level;
	private Integer children_level;
	private String format;
	private String provider;
	private String engine;

	private NodesService itemsService;
	private ModelService modelService;
	
	private final Injector injector;

	@Inject
	public GraphAPIService(Injector injector, ModelService modelService, NodesService itemsService, @Named("default.provider") String provider, @Named("default.engine") String engine) {
		this.itemsService = itemsService;
		this.id_center = -1;
		this.id_subgraph = -1;
		this.parents_level = 1;
		this.children_level = 1;
		this.format = "png";
		this.injector = injector;
		this.provider = provider;
		this.engine = engine;
		this.modelService = modelService;
	}
	
	@Override
	@Get
	public Reply<?> get() {
		return this.process();
	}
	
	@Override
	@Post
	public Reply<?> post() {
		return this.process();
	}

	private Reply<?> process() {
		String error = "";
		try {
			
			List<Node> nodes = new ArrayList<Node>();
			
			if(id_center.equals(-1)) {
				nodes = modelService.getView(id_subgraph).nodes();
			}
			else {
				Node center = itemsService.getNodeById(id_center);
				nodes.add(center);
				nodes.addAll(itemsService.getNodeParents(center, parents_level));
				nodes.addAll(itemsService.getNodeChildren(center, children_level));
			}
						
			GraphBuilder builder = GraphBuilder.graph(nodes).providedBy(provider).renderedBy(engine);
			
			if(format.equals("svg")) {
				return Reply.with(builder.asSVG(injector, URIContext.DISPLAY.replaceAll(":nodeId",""))).as(SVG.class);
			}
			
			return Reply.with(builder.asPNG(injector)).as(PNG.class);

		}
		catch (NodeNotFoundException e) {
			error = e.getMessage();
		}
		catch (GraphException e) {
			error = e.getMessage();
		}
		catch (UnableToRetrieveViewsException e) {
			error = e.getMessage();
		}
		catch (UnableToRetrieveNodesException e) {
			error = e.getMessage();
		}
		catch (ViewNotFoundException e) {
			error = e.getMessage();
		}
		
		return Reply.with(new ErrorReply(error)).as(Json.class);
	}
	
	public Integer getId_subgraph() {
		return id_subgraph;
	}

	public void setId_subgraph(Integer id_subgraph) {
		this.id_subgraph = id_subgraph;
	}

	public Integer getId_center() {
		return id_center;
	}

	public void setId_center(Integer id_center) {
		this.id_center = id_center;
	}

	public Integer getParents_level() {
		return parents_level;
	}

	public void setParents_level(Integer parents_level) {
		this.parents_level = parents_level;
	}

	public Integer getChildren_level() {
		return children_level;
	}

	public void setChildren_level(Integer children_level) {
		this.children_level = children_level;
	}

	public String getFormat() {
		return format;
	}

	public void setFormat(String format) {
		this.format = format;
	}

	public String getProvider() {
		return provider;
	}

	public void setProvider(String provider) {
		this.provider = provider;
	}

	public String getEngine() {
		return engine;
	}

	public void setEngine(String engine) {
		this.engine = engine;
	}
	
}
